/*
 * Hamsam - Instant Messaging API
 * Copyright (C) 2003 Raghu K
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package hamsam.api;

import javax.swing.Icon;

/**
 * A component to represent smileys (emoticons) in instant messages.
 *
 * <p>
 * A smiley is a small picture, sometimes referred to as an emoticon, used to represent
 * a user's emotions. For each smiley, there will be one or more alternate text
 * representations, and a descriptive name.
 *
 * <p>
 * For example, for a smiley showing a "sad face", the textual representation may be
 * something like ":(" or ":-(", while the descriptive name may be "sad".
 *
 * <p>
 * Clients that don't want to display smileys (may be because the client is a text
 * based one), may use the text representation. Alternatively, they may use the descriptive
 * name to convey the exact mood represented by the smiley.
 *
 */
public class SmileyComponent implements MessageComponent
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 5362092125523488052L;

	/**
	 * The icon ofr this smiley.
	 */
	private Icon icon;

	/**
	 * Alternate textual representation for this smiley.
	 */
	private String[] text;

	/**
	 * A descriptive name for this smiley.
	 */
	private String name;

	/**
	 * Construct a new smiley with a given icon and alternate text.
	 *
	 * @param icon the icon for this smiley.
	 * @param text the alternate text for this smiley.
	 * @param name the descriptive name for this smiley.
	 */
	public SmileyComponent(Icon icon, String[] text, String name)
	{
		this.icon = icon;
		this.text = text;
		this.name = name;
	}

	/**
	 * Returns the icon of this smiley.
	 *
	 * @return icon representing this smiley.
	 */
	public Icon getIcon()
	{
		return icon;
	}

	/**
	 * Returns the alternate text for this smiley.
	 *
	 * @return alternate text representing this smiley.
	 */
	public String[] getText()
	{
		return text;
	}

	/**
	 * Returns the descriptive name for this smiley.
	 *
	 * @return descriptive name representing this smiley.
	 */
	public String getName()
	{
		return name;
	}
}
