/*
 * Hamsam - Instant Messaging API
 * Copyright (C) 2003 Raghu K
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package hamsam.api;

import java.awt.Font;
import java.awt.Color;

/**
 * A text component of a message.
 *
 * <p>
 * A text component contains a sequence of characters that represent a message.
 * This sequence, together with the font and color information, determines what
 * to display to the user.
 *
 * <p>
 * This class defines various methods to set and retrieve the sequence of characters,
 * as well as the font and color information.
 *
 * @author  Raghu
 */
public class TextComponent implements MessageComponent
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 4404275548397146188L;

	/**
	 * Buffer holding the sequence of characters in this
	 * text component.
	 */
	private StringBuffer buffer;

	/**
	 * The font to be used for this component.
	 */
	private Font font;

	/**
	 * The color to be used for this component.
	 */
	private Color color;

	/**
	 * Construct a new text component that represents an empty character
	 * sequence. This constructor is equivalent to
	 * <code>TextComponent(null, null, null)</code>.
	 */
	public TextComponent()
	{
		this(null, null, null);
	}

	/**
	 * Construct a new text component and initialize it with the given
	 * character sequence. This constructor is equivalent to
	 * <code>TextComponent(str, null, null)</code>.
	 *
	 * @param str sequence of characters representing this text component.
	 */
	public TextComponent(String str)
	{
		this(str, null, null);
	}

	/**
	 * Construct a new text component and initialize it with a given
	 * character sequence and color information. This constructor is
	 * equivalent to <code>TextComponent(str, null, color)</code>.
	 *
	 * @param str sequence of characters representing this text component.
	 * @param color the color of the text represented by this component.
	 */
	public TextComponent(String str, Color color)
	{
		this(str, null, color);
	}

	/**
	 * Construct a new text component and initialize it with a given
	 * character sequence and font information. This constructor is
	 * equivalent to <code>TextComponent(str, font, null)</code>.
	 *
	 * @param str sequence of characters representing this text component.
	 * @param font the font of the text represented by this component.
	 */
	public TextComponent(String str, Font font)
	{
		this(str, font, null);
	}

	/**
	 * Construct a new text component and initialize it with a given
	 * character sequence, font, and color information.
	 *
	 * @param str sequence of characters representing this text component.
	 * @param font the font of the text represented by this component.
	 * @param color the color of the text represented by this component.
	 */
	public TextComponent(String str, Font font, Color color)
	{
		buffer = new StringBuffer();

		if(str != null)
			append(str);
		if(font != null)
			setFont(font);
		if(color != null)
			setColor(color);
	}

	/**
	 * Appends the string to this text component.
	 *
	 * @param str the string to be appended.
	 */
	public void append(String str)
	{
		buffer.append(str);
	}

	/**
	 * Appends the sequence of characters to this text component.
	 *
	 * @param str the characters to be appended.
	 */
	public void append(char[] str)
	{
		buffer.append(str);
	}

	/**
	 * Returns the sequence of characters corresponding to this
	 * text component.
	 *
	 * @return the sequence of characters of this component.
	 */
	public char[] getSequence()
	{
		return buffer.toString().toCharArray();
	}

	/**
	 * Specify the font to be used with this text component.
	 *
	 * @param font the font to be used.
	 */
	public void setFont(Font font)
	{
		this.font = font;
	}

	/**
	 * Returns the font that is used with this text component.
	 *
	 * @return the font used.
	 */
	public Font getFont()
	{
		return font;
	}

	/**
	 * Specify the color to be used with this text component.
	 *
	 * @param color the color to be used.
	 */
	public void setColor(Color color)
	{
		this.color = color;
	}

	/**
	 * Returns the color that is used with this text component.
	 *
	 * @return the color used.
	 */
	public Color getColor()
	{
		return color;
	}
}
